#include <conio.h>
#include <dos.h>
#include <sys/timeb.h>
#include "events.h"

#define MOUSE_INT	0x33
#define LEFT		1
#define RIGHT		2
#define MOUSE_DELAY	100

int	_mouse_installed;
int	_mouse_delay = MOUSE_DELAY;

extern PANEL	_panels[MAXPANELS];
extern BUTTON	_buttons[MAXBUTTONS];

int	_lastpanel;
int	_lastbutton;

void	(*dfl_event)() = (void (*)()) 0L;
void	(*timer_event)() = (void (*)()) 0L;


int _mouse_detect()
{
   union REGS  regs;

   regs.x.ax = 0;
   int86(MOUSE_INT, &regs, &regs);
   return(regs.x.ax == 0 ? -1 : 0);
}


void mouse_on()
{
   union REGS  regs;

   regs.x.ax = 1;
   int86(MOUSE_INT, &regs, &regs);
}


void mouse_off()
{
   union REGS  regs;

   regs.x.ax = 2;
   int86(MOUSE_INT, &regs, &regs);
}


int _mouse_button()
{
   union REGS  regs;

   if (!_mouse_installed) return(0);
   regs.x.ax = 3;
   int86(MOUSE_INT, &regs, &regs);
   return(regs.h.bl);
}


void _get_mousepos(x, y)
int  *x, *y;
{
   union REGS  regs;

   regs.x.ax = 3;
   int86(MOUSE_INT, &regs, &regs);
   *x = regs.x.cx;
   *y = regs.x.dx;
}


void _set_mousepos(x, y)
int  x, y;
{
   union REGS  regs;

   regs.x.ax = 4;
   regs.x.cx = x;
   regs.x.dx = y;
   int86(MOUSE_INT, &regs, &regs);
}


void _set_mousearea(x, y, dx, dy)
int  x, y, dx, dy;
{
   union REGS  regs;

   regs.x.ax = 7;
   regs.x.cx = x;
   regs.x.dx = dx;
   int86(MOUSE_INT, &regs, &regs);
   regs.x.ax = 8;
   regs.x.cx = y;
   regs.x.dx = dy;
   int86(MOUSE_INT, &regs, &regs);
   _set_mousepos(x, y);
}


void _mouse_hardcsr()
{
   union REGS  regs;
   
   regs.x.ax = 10;
   regs.x.bx = 1;
   regs.x.cx = 0;
   regs.x.dx = 0;
   int86(MOUSE_INT, &regs, &regs);
}


void _mouse_softcsr(csr, scr)
int  csr, scr;
{
   union REGS  regs;
   
   regs.x.ax = 10;
   regs.x.bx = 0;
   regs.x.cx = scr;
   regs.x.dx = csr;
   int86(MOUSE_INT, &regs, &regs);
}


void init_mouse(x, y, dx, dy)
{
   int  i;

   if (dx < x) {
	i = x; x = dx; dx = i;
   }
   if (dy < y) {
	i = y; y = dy; dy = i;
   }
   _set_mousearea(x, y, dx, dy);
   _set_mousepos(x+(dx-x)/2, y+(dy-y)/2);
}


int event_init()
{
   if (_mouse_detect() < 0)
	_mouse_installed = 0;
   else
	_mouse_installed = 1;

   _lastpanel = _lastbutton = 0;

   delay(0);

   return(_mouse_installed);
}


void clearpanels()
{
   _lastpanel = 0;
}


int addpanel(x, y, dx, dy, enter, leave)
int  x, y, dx, dy;
void (*enter)(), (*leave)();
{
   if (_lastpanel >= MAXPANELS)
	return(-1);
   _panels[_lastpanel].x = x;
   _panels[_lastpanel].y = y;
   _panels[_lastpanel].dx = dx;
   _panels[_lastpanel].dy = dy;
   _panels[_lastpanel].enter = enter;
   _panels[_lastpanel].leave = leave;
   return(++_lastpanel);
}


void clearbuttons()
{
   _lastbutton = 0;
}


int addbutton(x, y, dx, dy, key, rpt, right)
int  x, y, dx, dy;
int  key, rpt, right;
{
   if (_lastbutton >= MAXBUTTONS)
	return(-1);
   _buttons[_lastbutton].x = x;
   _buttons[_lastbutton].y = y;
   _buttons[_lastbutton].dx = dx;
   _buttons[_lastbutton].dy = dy;
   _buttons[_lastbutton].key = key;
   _buttons[_lastbutton].rpt = rpt;
   _buttons[_lastbutton].right = right;
   return(++_lastbutton);
}


void _mouse_panel()
{
   int  mx, my, i;
   static int  px = -1, py = -1;

   _get_mousepos(&mx, &my);
   for (i=0; i<_lastpanel; i++) {
	if (mx >= _panels[i].x && mx <= _panels[i].dx &&
	my >= _panels[i].y && my <= _panels[i].dy &&
	(px < _panels[i].x || px > _panels[i].dx ||
	py < _panels[i].y || py > _panels[i].dy)) {
	   mouse_off();
	   (*_panels[i].enter)();
	   mouse_on();
	   /*return;*/
	}
	if (px >= _panels[i].x && px <= _panels[i].dx &&
	py >= _panels[i].y && py <= _panels[i].dy &&
	(mx < _panels[i].x || mx > _panels[i].dx ||
	my < _panels[i].y || my > _panels[i].dy)) {
	   mouse_off();
	   (*_panels[i].leave)();
	   mouse_on();
	   /*return;*/
	}
   }
   px = mx; py = my;
}


int mouse_key()
{
   int  mx, my, i;
   int  right;

   if (_mouse_button() == 1)
	right = 0;
   else
	right = 1;

   _get_mousepos(&mx, &my);
   for (i=0; i<_lastbutton; i++)
	if (mx >= _buttons[i].x && mx <= _buttons[i].dx &&
	my >= _buttons[i].y && my <= _buttons[i].dy &&
	_buttons[i].right == right) {
	   if (_buttons[i].rpt)
		delay(_mouse_delay);
	   else
		while (_mouse_button())
		   ;
	   return(_buttons[i].key);
	}
   if (dfl_event != (void (*)()) 0L)
	(*dfl_event)();
   return(0);
}


int getevent()
{
   int  c;

   if (_mouse_installed)
	mouse_on();
   for (;;) {
	if (timer_event != (void (*)()) 0L)
	   (*timer_event)();
	if (_mouse_installed)
	   _mouse_panel();
	if (_mouse_installed)
	   if (_mouse_button()) {
		mouse_off();
		return(mouse_key());
	   }
	if (kbhit()) {
	   if (_mouse_installed)
		mouse_off();
	   if ((c = getch()) == 0)
		return(ext(getch()));
	   return(c);
	}
   }
}
