#include <stdio.h>
#include <string.h>
#include <alloc.h>
#include <sys/stat.h>

#define MAGIC	"NMH"
#define BUFSIZE	4096
#undef FAST


#ifdef STANDALONE
void fatal(m, n)
char  *m, *n;
{
   fprintf(stderr, "%s: %s\n", m, n);
   exit(1);
}
#endif


#ifdef FAST

int unsqueeze(name, name2)
char  *name;
char  *name2;
{
   char  iname[81], oname[81], magic[4];
   unsigned char  *ibuf, *obuf;
   FILE  *in, *out;
   int   c, i, count;
   unsigned  ip, op;
   struct stat  st;

#ifdef STANDALONE
   sprintf(iname, "%s.bbz", name);
   sprintf(oname, "%s.bbl", name);
#else
   strcpy(iname, name);
   strcpy(oname, name2);
#endif

   if (stat(iname, &st) < 0)
#ifdef STANDALONE
	fatal("Cannot stat file", iname);
#else
	return(-1);
#endif

   if ((in = fopen(iname, "rb")) == NULL)
#ifdef STANDALONE
	fatal("Cannot open file", iname);
#else
	return(-1);
#endif

   if ((ibuf = (unsigned char *) malloc((unsigned) st.st_size)) == NULL) {
	fclose(in);
#ifdef STANDALONE
	fatal("Out of memory");
#else
	return(-4);
#endif
   }

   if ((obuf = (unsigned char *) malloc((unsigned) 65535u)) == NULL) {
	fclose(in);
	free(ibuf);
#ifdef STANDALONE
	fatal("Out of memory");
#else
	return(-4);
#endif
   }

   fgets(magic, 4, in);
   if (strcmp(magic, MAGIC)) {
	free(ibuf);
	free(obuf);
	fclose(in);
#ifdef STANDALONE
	fatal("Bad magic ID", magic);
#else
	return(-2);
#endif
   }

   if ((out = fopen(oname, "wb")) == NULL) {
	free(ibuf);
	free(obuf);
	fclose(in);
#ifdef STANDALONE
	fatal("Cannot create file", oname);
#else
	return(-1);
#endif
   }

   fread(ibuf, 1, (unsigned) st.st_size, in);
   ip = op = 0;

   count = ibuf[ip++];
   for (;;) {
	if (ip >= (unsigned) st.st_size)
	   break;
	c = ibuf[ip++];
	for (i=0; i<count; i++)
	   obuf[op++] = c;
	count = ibuf[ip++];
   }

   fwrite(obuf, 1, op, out);

   fclose(in);
   fclose(out);
   free(ibuf);
   free(obuf);

   return(0);
}

#else	/* FAST */

int unsqueeze(name, name2)
char  *name;
char  *name2;
{
   char  iname[81], oname[81], magic[4];
   FILE  *in, *out;
   int   c, i, count;

#ifdef STANDALONE
   sprintf(iname, "%s.bbz", name);
   sprintf(oname, "%s.bbl", name);
#else
   strcpy(iname, name);
   strcpy(oname, name2);
#endif

   if ((in = fopen(iname, "rb")) == NULL)
#ifdef STANDALONE
	fatal("Cannot open file", iname);
#else
	return(-1);
#endif

   fgets(magic, 4, in);
   if (strcmp(magic, MAGIC)) {
	fclose(in);
#ifdef STANDALONE
	fatal("Bad magic ID", magic);
#else
	return(-2);
#endif
   }

   if ((out = fopen(oname, "wb")) == NULL) {
	fclose(in);
#ifdef STANDALONE
	fatal("Cannot create file", oname);
#else
	return(-1);
#endif
   }

   setvbuf(in, NULL, _IOFBF, BUFSIZE);
   setvbuf(out, NULL, _IOFBF, BUFSIZE);

   count = fgetc(in);
   for (;;) {
	if (feof(in))
	   break;
	c = fgetc(in);
	for (i=0; i<count; i++)
	   fputc(c, out);
	count = fgetc(in);
   }

   fclose(in);
   fclose(out);

   return(0);
}

#endif	/* FAST */


#ifdef STANDALONE
int squeeze(name)
char  *name;
{
   char  iname[81], oname[81];
   FILE  *in, *out;
   int   p, c, count;

   sprintf(iname, "%s.bbl", name);
   sprintf(oname, "%s.bbz", name);

   if ((in = fopen(iname, "rb")) == NULL)
	fatal("Cannot open file", iname);

   if ((out= fopen(oname, "wb")) == NULL) {
	fclose(in);
	fatal("Cannot open file", oname);
   }

   setvbuf(in, NULL, _IOFBF, 4096);
   setvbuf(out, NULL, _IOFBF, 4096);

   fputs(MAGIC, out);
   p = fgetc(in);
   for (;;) {
	c = fgetc(in);
	if (feof(in))
	   break;
	count = 1;
	while (p == c && count < 255 && !feof(in)) { 
	   count++;
	   c = fgetc(in);
	}
	fputc(count, out);
	fputc(p, out);
	p = c;
   }

   fclose(in);
   fclose(out);

   return(0);
}


void usage()
{
   fprintf(stderr, "Usage: bbsq [-d] file\n");
}


int main(argc, argv)
int   argc;
char  **argv;
{
   int  p;
   int  decode = 0;

   for (p = 1; argv[p][0] == '-'; p++)
	for (++argv[p]; *argv[p]; argv[p]++)
	   switch (*argv[p]) {
		case 'd':	decode = 1; break;
		default:	usage(); exit(1);
	   }

   if (p >= argc) {
	usage();
	exit();
   }

   if (decode)
	unsqueeze(argv[p], NULL);
   else
	squeeze(argv[p]);

   return(0);
}
#endif
